from .logging import get_logger
from .base import Base

import numpy as np
import pandas as pd
import tensorflow as tf

logger = get_logger()


def isEnglish(s):
    try:
        s.encode(encoding="utf-8").decode("ascii")
    except UnicodeDecodeError:
        return False
    else:
        return True


class Naam(Base):
    MODELFN = "model"
    weights_loaded = False
    model = None
    classes = ["not-muslim", "muslim"]
    cur_lang = "eng"
    model_name = "eng_and_hindi_models_v1"

    @classmethod
    def pred_rel(cls, input, lang="eng", latest=False):
        """
        Predict religion based on name
        Args:
            input (str): Name in Hindi Or English text
        Returns:
            output (str): religion name
        """

        if not cls.weights_loaded or cls.cur_lang != lang:
            cls.model_path = cls.load_model_data(cls.model_name, latest)
            if lang == "eng":
                cls.model = tf.keras.models.load_model(f"{cls.model_path}/{cls.model_name}/eng_model")
            else:
                cls.model = tf.keras.models.load_model(f"{cls.model_path}/{cls.model_name}/hin_model")
            cls.weights_loaded = True
            cls.cur_lang = lang

        results = cls.model.predict(input)
        res_args = tf.argmax(results, 1)
        probs = tf.nn.softmax(results)

        labels = []
        muslim_probs = []
        for i in range(0, len(input)):
            labels.append(cls.classes[res_args[i]])
            muslim_probs.append(np.around(probs[i][1] * 100))
        return pd.DataFrame(data={"name": input, "pred_label": labels, "pred_prob_muslim": muslim_probs})



import sys
import argparse

from .naam import Naam

pred_rel = Naam.pred_rel


def main(argv=sys.argv[1:]):
    title = "Predict religion based on name"
    parser = argparse.ArgumentParser(description=title)
    parser.add_argument("--input", default=None, help="name")
    args = parser.parse_args(argv)
    print(args)
    if not args.input or not args.type:
        return -1

    output = pred_rel(args.input)
    print(output)

    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for pranaam.py

"""

import unittest
import pandas as pd
from pranaam import pranaam


class TestPredRel(unittest.TestCase):
    def setUp(self):
        eng_names = [
            {"name": "Shah Rukh Khan", "true_rel": "muslim"},
            {"name": "Amitabh Bachchan", "true_rel": "not-muslim"},
        ]
        self.eng_df = pd.DataFrame(eng_names)

        hin_names = [
            {"name": "शाहरुख खान", "true_rel": "muslim"},
            {"name": "अमिताभ बच्चन", "true_rel": "not-muslim"},
        ]
        self.hin_df = pd.DataFrame(hin_names)

    def tearDown(self):
        pass

    def test_pred_label(self):
        odf = pranaam.pred_rel(self.eng_df["name"])
        self.assertIn("pred_label", odf.columns)
        self.assertTrue(odf.iloc[0]["pred_label"] == self.eng_df.iloc[0]["true_rel"])
        self.assertTrue(odf.iloc[1]["pred_label"] == self.eng_df.iloc[1]["true_rel"])

    def test_pred_prob_muslim(self):
        odf = pranaam.pred_rel(self.eng_df["name"])
        self.assertIn("pred_prob_muslim", odf.columns)

    def test_hindi(self):
        odf = pranaam.pred_rel(self.hin_df["name"], lang="hin")
        self.assertIn("pred_label", odf.columns)
        self.assertTrue(odf.iloc[0]["pred_label"] == self.hin_df.iloc[0]["true_rel"])
        self.assertTrue(odf.iloc[1]["pred_label"] == self.hin_df.iloc[1]["true_rel"])


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

import os
import tarfile
import requests
from .logging import get_logger

logger = get_logger()

REPO_BASE_URL = os.environ.get("PRANAAM_MODEL_URL") or "https://dataverse.harvard.edu/api/access/datafile/6286241"


def download_file(url, target, file_name):
    status = True
    file_path = f"{target}/{file_name}.tar.gz"
    try:
        print("Download models from dataverse...")
        # download the file
        r = requests.get(url, allow_redirects=True)
        open(file_path, "wb").write(r.content)
        # untar
        with tarfile.open(file_path, "r:gz") as tar_ref:
            tar_ref.extractall(target)
        # remove zip file
        os.remove(file_path)
        print("Finished downloading models ...")
    except Exception as exe:
        logger.error(f"Not able to download models {exe}")
        status = False
    return status



import os
from pkg_resources import resource_filename
from .utils import download_file, REPO_BASE_URL
from .logging import get_logger

logger = get_logger()


class Base(object):
    MODELFN = None

    @classmethod
    def load_model_data(cls, file_name, latest=False):
        model_path = None
        if cls.MODELFN:
            model_fn = resource_filename(__name__, cls.MODELFN)
            print(f"Model path {model_fn}")
            if not os.path.exists(model_fn):
                os.makedirs(f"{model_fn}")
            if not os.path.exists(f"{model_fn}/{file_name}") or latest:
                logger.debug(
                    "Downloading model data from the server (this is done only first time) ({0!s})...".format(model_fn)
                )
                if not download_file(REPO_BASE_URL, f"{model_fn}", file_name):
                    logger.error("ERROR: Cannot download model data file")
            else:
                logger.debug("Using model data from {0!s}...".format(model_fn))
            model_path = model_fn

        return model_path
